#include "AneuMeshLoader.h"
#include "Exception.h"
#include <fstream>
#include <iostream>

void AneuMeshLoader::LoadMesh(const std::string& filename) {
    std::ifstream fin(filename);
    if (!fin) {
        throw NoFileFoundException(filename);
    }
    else std::cout << "File \"" << filename << "\" was opened successfully." << std::endl;

    int node_amount, dimension,elem_amount,bound_amount,nodes_in_elem, nodes_in_bound;

    fin >> node_amount >> dimension;
    nodes.reserve(node_amount);
    for (int i = 1; i <= node_amount; ++i) {
        Node temp_node;
        temp_node.id = i;
        fin >> get<0>(temp_node.ords) >> get<1>(temp_node.ords) >> get<2>(temp_node.ords);
        temp_node.vertex = true;
        nodes.push_back(temp_node);
    }

    fin >> elem_amount >> nodes_in_elem;
    elements.reserve(elem_amount);
    for (int i = 0; i <= elem_amount; ++i) {
        Element temp_element;
        temp_element.element_id = i+1;
        fin >> temp_element.material_id;
        temp_element.node_id.reserve(nodes_in_elem);
        
        for (int j = 0; j < nodes_in_elem; ++j) {
            int temp_id;
            fin >> temp_id;
            temp_element.node_id.push_back(temp_id);
        }
        elements.push_back(std::move(temp_element));
    }

    fin >> bound_amount >> nodes_in_bound;
    boundaries.reserve(bound_amount);
    for (int i = 0; i < bound_amount; ++i) {
        ElemBoundary bound_temp;
        bound_temp.element_id = i + 1;
        fin >> bound_temp.surface_id;
        bound_temp.node_id.reserve(nodes_in_bound);
        for (int j = 0; j < nodes_in_bound; ++j) {
            int node_id;
            fin >> node_id;
            nodes[node_id - 1].vertex = true;
            bound_temp.node_id.push_back(node_id);
        }
        boundaries.push_back(std::move(bound_temp));  

    }

}
