#include "DataStructures.h"
#include <iterator>
#include <optional>


std::optional<int> readInt(std::istream& in) {
	int i;
	if (in >> i)
	{
		return { i };
	}
	return {};
}

//--------------------------- Node ------------------------------
Node::Node(int p_id, const std::array<double, 3>& p_ords, bool p_vertex)
	: id(p_id)
	, ords(p_ords)
	, vertex(p_vertex)
{}

Node::Node(const Node& p_node)
	: id(p_node.id)
	, ords(p_node.ords)
	, vertex(p_node.vertex)
{}



std::istream& operator >> (std::istream& p_in, Node& p_node)
{
	p_in >> p_node.id;
	p_in >> get<0>(p_node.ords)
		>> get<1>(p_node.ords)
		>> get<2>(p_node.ords);
	p_in >> p_node.vertex;
	return p_in;
}

std::ostream& operator<< (std::ostream& p_out, const Node& p_node)
{
	p_out << p_node.id;
	p_out << get<0>(p_node.ords)
		<< get<1>(p_node.ords)
		<< get<2>(p_node.ords);
	p_out << p_node.vertex;
	return p_out;
}

//--------------------------- Element ------------------------------

std::istream& operator>>(std::istream& p_in, Element& p_elem)
{
	p_in >> p_elem.element_id
		>> p_elem.material_id;

	std::optional<int> p_node_id;
	p_node_id = readInt(p_in);
	while (p_node_id) {
		p_elem.node_id.push_back(*p_node_id);
		p_node_id = readInt(p_in);
	}
	return p_in;
}

std::ostream& operator<<(std::ostream& p_out, const Element& p_finite_elem)
{
	p_out << p_finite_elem.element_id << std::endl
		<< p_finite_elem.material_id << std::endl;
	p_out << "[ ";
	for (const auto& elem : p_finite_elem.node_id)
		p_out << elem << " ";
	p_out << "]";
	return p_out;
}

Element::Element(int p_element_id, int p_material_id, const std::vector<int>& p_node_id)
	: element_id(p_element_id)
	, material_id(p_material_id)
	, node_id(p_node_id)
{}

//--------------------------- EleBoundary ------------------------------

std::istream& operator>>(std::istream& p_in, ElemBoundary& p_bound_finite_elem)
{
	p_in >> p_bound_finite_elem.surface_id
		>> p_bound_finite_elem.element_id;

	std::optional<int> node_id;
	node_id = readInt(p_in);
	while (node_id) {
		p_bound_finite_elem.node_id.push_back(*node_id);
		node_id = readInt(p_in);
	}
	return p_in;
}

std::ostream& operator<<(std::ostream& p_out, const ElemBoundary& p_bound_finite_elem)
{
	p_out << p_bound_finite_elem.surface_id << std::endl
		<< p_bound_finite_elem.element_id << std::endl;
	p_out << "[ ";
	for (const auto& elem : p_bound_finite_elem.node_id)
		p_out << elem << " ";
	p_out << "]";
	return p_out;
}

ElemBoundary::ElemBoundary(int p_element_id, int p_bound_id, const std::vector<int>& p_node_id)
	: element_id(p_element_id)
	, surface_id(p_bound_id)
	, node_id(p_node_id)
{}

void Edge::update_mid(int p_middle_node)
{
	middle_node = p_middle_node;
}
Edge::Edge(int p_id_first_node, int p_id_second_node)
	: nodes(std::make_pair(p_id_first_node, p_id_second_node))
{
	middle_node = 0;
}

Edge Edge::operator=(const Edge& p_edge)
{
	Edge res;
	res.nodes = p_edge.nodes;

	return res;
}

bool Edge::operator==(const Edge& p_edge) const
{
	return (nodes.first == p_edge.nodes.first)
		&& (nodes.second == p_edge.nodes.second);
}

Edge Edge::create_edge(int p_first_node_id, int p_second_node_id)
{
	return Edge(p_first_node_id, p_second_node_id);
}



std::istream& operator>>(std::istream& p_in, Edge& p_edge)
{
	p_in >> p_edge.nodes.first >> p_edge.nodes.second;
	return p_in;
}

std::ostream& operator<<(std::ostream& p_out, Edge& p_edge)
{
	p_out << "[" << p_edge.nodes.first << ", " << p_edge.nodes.second << "]";
	return p_out;
}

